

#ifndef _RATEPLUG_CONFIG_H_
#define _RATEPLUG_CONFIG_H_

#include <gtk/gtk.h>
#include <stdlib.h>
#include <stdio.h>
#include <string>

// xmms includes
#include <xmms/configfile.h>
#include <xmms/util.h>
#include <xmms/xmmsctrl.h>

//my includes
#include "database.h"

#define CONFIGOK 1
#define CONFIGERROR -1
#define CONFIGCANNOTREAD -1000
#define CONFIGCANNOTOPEN -1001

#define DEFAULT_DATABASELOCATION  "~/.xmms/database.txt"
#define DEFAULT_TIMEOUT 50
#define DEFAULT_ASSESSMENTSONGHEARED 10
#define DEFAULT_ASSESSMENTSONGSKIPPED 10


class Config {

  GtkWidget *configWin;
  GtkWidget *inputSkipSong, *inputHearedSong, *inputDatabaseLocation,
            *inputTimeout, *inputCurrentSong ,*inputCurrentSongAssessment, 
            *inputNewAssessment;
  ConfigFile *confFile;
  int iSongsRatedCountUntilNow;
  // used in ignoreCurrentSong
  Database::ASCIIDatabase* db;
  GeneralPlugin *plugin;




 public:
  int iRatingSongSkiped, iRatingSongHeared, iSongsRatedCount,
      iTimeout;
  char* cDatabaseLocation;
  




 public:

  // a little bit of bungling 
  Config(GeneralPlugin *_plugin, Database::ASCIIDatabase *_db) {
    plugin = _plugin;
    db = _db;
    iRatingSongSkiped = DEFAULT_ASSESSMENTSONGSKIPPED;
    iRatingSongHeared = DEFAULT_ASSESSMENTSONGHEARED;
    iTimeout = DEFAULT_TIMEOUT;
    iSongsRatedCountUntilNow = 0;
    iSongsRatedCount = 0;
    cDatabaseLocation = DEFAULT_DATABASELOCATION;
    configWin = NULL;
    inputSkipSong = NULL;
    inputHearedSong = NULL;
    inputDatabaseLocation = NULL;
    inputTimeout = NULL;
    inputCurrentSong = NULL;
    inputCurrentSongAssessment = NULL;
    inputNewAssessment = NULL;

    confFile = NULL;

    configWin = NULL;

  }

  ~Config(){
  }
  

// windows
  // create the default configure window
  int createConfigWindow();
     // callback
     void configWindowOKButtonClicked();
     void configWindowCancelButtonClicked();
     void configWindowRateButtonClicked();

  // refresh configure window
     int refreshConfigWindow(int position);


  void createRestoreReallyWindow();
  // restore all config values to default
  void restoreDefaultValues();

  void createFileNotFound();

  void createAboutWindow();

  // save config values
  int saveValues();

  int readValues();


  int ignoreCurrentSong();

 private:
  void createValidValuesWindow();

// functions
  // tests all config values for correct values  
  int checkValues();

};// end class Config

static void st_configWindowRateButtonClicked(GtkWidget *widget, Config *object) {
  object->configWindowRateButtonClicked();
}

static void st_configWindowOKButtonClicked(GtkWidget *widget, Config *object){
  object->configWindowOKButtonClicked();
}
static void st_configWindowCancelButtonClicked(GtkWidget *widget, Config *object) {
  object->configWindowCancelButtonClicked();
}

static void st_createRestoreReallyWindow(GtkWidget *widget, Config *object){
  object->createRestoreReallyWindow();
}
static void st_restoreDefaultValues(GtkWidget *widget, Config *object){
  object->restoreDefaultValues();
}
static void st_ignoreCurrentSong(GtkWidget *widget, Config *object){
  object->ignoreCurrentSong();
}

/*
 create a nice Configure Window
*/
int Config::createConfigWindow(){
  GtkWidget *label, *hbox, *hbox2, *vbox, *vbox2, *frame, *button;
   GtkWidget *table;
   //   ConfigFile *confFile
   // read conf help variables 
   //   gchar *cReadSOS, *cReadSTS, *cReadSS;
   char *chelp;
   chelp = (char*)malloc(sizeof(int)*8+1);


   if(configWin != NULL){

     return 0;
   }

   readValues();


   // let's create a nice configure window...  ;)
   configWin = gtk_window_new(GTK_WINDOW_DIALOG);
   gtk_signal_connect(GTK_OBJECT(configWin), "destroy", GTK_SIGNAL_FUNC(gtk_widget_destroyed), &configWin);//coders friend: copy&paste....
   gtk_window_set_title(GTK_WINDOW(configWin), ("Song Rating Plugin Configuration"));
   gtk_container_set_border_width (GTK_CONTAINER (configWin), 15);
   
   // okay....
   vbox = gtk_vbox_new (FALSE, 10);
   gtk_container_add (GTK_CONTAINER (configWin), vbox);

   // every line a new hbox.......
   
  // yes yes yoooo...    
   // some hot text ;)
   //line 1
   hbox = gtk_hbox_new(FALSE, 0);
   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
   label = gtk_label_new   ("song skiped completly (decrease assessment): ");
   gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
 // input line 
   inputSkipSong = gtk_entry_new_with_max_length(4);
   gtk_widget_set_usize(inputSkipSong, 50, -1);
   sprintf(chelp, "%d", iRatingSongSkiped);
   gtk_entry_set_text (GTK_ENTRY (inputSkipSong), chelp);
   gtk_entry_select_region (GTK_ENTRY (inputSkipSong), 0, GTK_ENTRY(inputSkipSong)->text_length);
   gtk_box_pack_end(GTK_BOX(hbox), inputSkipSong ,FALSE, FALSE, 0);

   //local showtime
   gtk_widget_show(inputSkipSong);
   gtk_widget_show(label);
   gtk_widget_show(hbox);

   //line 2
   hbox = gtk_hbox_new(FALSE, 10);
   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
   //text
   label = gtk_label_new(("song heared completly (increase assessment): "));
   gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
   gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
   // input line 
   inputHearedSong = gtk_entry_new_with_max_length(4);
   gtk_widget_set_usize(inputHearedSong, 50, -1);
   sprintf(chelp, "%d", iRatingSongHeared);
   gtk_entry_set_text (GTK_ENTRY (inputHearedSong),chelp);
   gtk_entry_select_region (GTK_ENTRY (inputHearedSong), 0, GTK_ENTRY(inputHearedSong)->text_length);
   gtk_box_pack_end(GTK_BOX(hbox), inputHearedSong ,FALSE, FALSE, 0);

   //local showtime
   gtk_widget_show(inputHearedSong);
   gtk_widget_show(label);
   gtk_widget_show(hbox);
   
   // line: set timeout
   hbox = gtk_hbox_new(FALSE, 10);
   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
   
   label = gtk_label_new("set the timeout (ms): ");
   gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
   gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
   // input line
   inputTimeout = gtk_entry_new_with_max_length(4);
   gtk_widget_set_usize(inputTimeout, 50, -1);
   sprintf(chelp, "%u", iTimeout);
   gtk_entry_set_text(GTK_ENTRY(inputTimeout), chelp);
   gtk_entry_select_region(GTK_ENTRY(inputTimeout), 0, GTK_ENTRY(inputTimeout)->text_length);
   gtk_box_pack_end(GTK_BOX(hbox), inputTimeout, FALSE, FALSE, 0);
   
   gtk_widget_show(label);
   gtk_widget_show(inputTimeout);
   gtk_widget_show(hbox);

   //line set database location
   table =gtk_table_new(1, 2, FALSE);
   gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, 0);


   label = gtk_label_new(("database location: "));
   gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
   gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
   gtk_widget_show(label);

 // input line 
   inputDatabaseLocation = gtk_entry_new_with_max_length(256);

   gtk_entry_set_text (GTK_ENTRY (inputDatabaseLocation), cDatabaseLocation);
   gtk_entry_select_region (GTK_ENTRY (inputDatabaseLocation), 0, GTK_ENTRY(inputDatabaseLocation)->text_length);

   gtk_table_attach_defaults(GTK_TABLE(table), inputDatabaseLocation, 1, 2, 0, 1);
   gtk_widget_show(table);


   //local showtime
   gtk_widget_show(inputDatabaseLocation);
   gtk_widget_show(hbox);


   frame = gtk_frame_new(" rate current song ");
   gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);
   gtk_widget_show(frame);


   vbox2 = gtk_vbox_new(FALSE, 10);
   gtk_container_set_border_width(GTK_CONTAINER(vbox2), 5);
   gtk_container_add(GTK_CONTAINER(frame), vbox2);


   table =gtk_table_new(4, 6, FALSE);
   gtk_box_pack_start(GTK_BOX(vbox2), table, FALSE, FALSE, 0);

   
   hbox2 = gtk_hbox_new(FALSE, 10);
   gtk_table_attach_defaults(GTK_TABLE(table), hbox2, 0, 1, 0, 1);
   label = gtk_label_new("current song: ");
   gtk_box_pack_start(GTK_BOX(hbox2), label, FALSE, FALSE, 0);
   gtk_widget_show(label);
   gtk_widget_show(hbox2);


   label = gtk_label_new("assessment:");
   gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_RIGHT);
   gtk_table_attach(GTK_TABLE(table), label, 2, 4, 0, 1, GTK_FILL, GTK_FILL, 0, 0);

   gtk_widget_show(label);
   gtk_widget_show(hbox2);

  
   // line 2
   inputCurrentSong = gtk_entry_new();
   gtk_widget_set_usize(inputCurrentSong, 200, -1);
   gtk_entry_set_text (GTK_ENTRY (inputCurrentSong), "current file");
   gtk_entry_set_editable(GTK_ENTRY(inputCurrentSong), FALSE);
   gtk_table_attach_defaults(GTK_TABLE(table), inputCurrentSong, 0, 1, 1, 2);
   gtk_widget_show(inputCurrentSong);

   inputCurrentSongAssessment = gtk_entry_new_with_max_length(4);
   gtk_widget_set_usize(inputCurrentSongAssessment, 50, -1);

   gtk_entry_set_editable(GTK_ENTRY(inputCurrentSongAssessment), FALSE);
   gtk_table_attach(GTK_TABLE(table), inputCurrentSongAssessment, 3, 4, 1, 2, GTK_FILL, GTK_FILL, 0, 0);

   gtk_widget_show(inputCurrentSongAssessment);
   gtk_widget_show(hbox2);

 
   // new assesment line
   button = gtk_button_new_with_label(" Rate current song with new value: ");
   gtk_table_attach_defaults(GTK_TABLE(table), button, 0, 1, 3, 4);
   gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(st_configWindowRateButtonClicked), this);
   gtk_widget_show(button);

   inputNewAssessment = gtk_entry_new_with_max_length(4);
   gtk_widget_set_usize(inputNewAssessment, 50, -1);
   gtk_table_attach(GTK_TABLE(table), inputNewAssessment, 3, 4, 3, 4, GTK_FILL, GTK_FILL, 0, 0);
   gtk_widget_show(inputNewAssessment);


   // ignore song button line
   button = gtk_button_new_with_label(" Ignore current song ");
   gtk_table_attach_defaults(GTK_TABLE(table), button, 0, 1, 5, 6);
   gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(st_ignoreCurrentSong), this);
   gtk_widget_show(button);

   


   gtk_widget_show(table);
   gtk_widget_show(vbox2);
   gtk_widget_show(frame);






   //button line
   // cause buttons should be centered (remind html??)
   hbox2 = gtk_hbox_new(TRUE, 0);
   gtk_box_pack_start(GTK_BOX(vbox), hbox2, FALSE, FALSE, 0);

   hbox = gtk_hbox_new(TRUE, 15);
   gtk_box_pack_start(GTK_BOX(hbox2), hbox, FALSE, FALSE, 0);

   button = gtk_button_new_with_label(" OK ");
   gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
   gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(st_configWindowOKButtonClicked), this);
   

   GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
   gtk_widget_show(button);
   gtk_widget_grab_default(button);

   button = gtk_button_new_with_label(" Cancel ");
   gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
   gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(st_configWindowCancelButtonClicked), this);
   GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
   gtk_widget_show(button);

   button = gtk_button_new_with_label(" Default ");
   gtk_box_pack_end(GTK_BOX(hbox), button, TRUE, TRUE, 0);
   gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(st_createRestoreReallyWindow), this);
   GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
   gtk_widget_show(button);

 
   //local showtime

   gtk_widget_show(hbox);


   //problem occurs while developing xmms plugins:
   //you must restart your xmms every few minutes (trial&error ;)
   //you need good, old CD ;)


   // show section
   gtk_widget_show(hbox2);
   gtk_widget_show(vbox);
   gtk_widget_show(configWin);
   // perhaps i've forgotten one
   gtk_widget_show_all (configWin);

   free (chelp);


   return CONFIGOK;
}// end configCreateWindow()



/*
  Callbacks - handle clicked buttons in config Window
*/
void Config::configWindowOKButtonClicked(){

  if (checkValues()) {
    saveValues();
    gtk_widget_destroy(GTK_WIDGET(configWin));
    // reset

    configWin = NULL;
    inputSkipSong = NULL;
    inputHearedSong = NULL;
    inputDatabaseLocation = NULL;
    inputTimeout = NULL;
      inputCurrentSong = NULL;
    inputCurrentSongAssessment = NULL;
    inputNewAssessment = NULL;
    
  }
  
}// end configWindowButtonClicked()



void Config::configWindowCancelButtonClicked(){

    gtk_widget_destroy(GTK_WIDGET(configWin));

    configWin = NULL;
    inputSkipSong = NULL;
    inputHearedSong = NULL;
    inputDatabaseLocation = NULL;
    inputTimeout = NULL;
    inputCurrentSong = NULL;
    inputCurrentSongAssessment = NULL;
    inputNewAssessment = NULL;

    
} // end configWindowCancelButtonClicked()



void Config::configWindowRateButtonClicked() {

  int iNew;
  string strhelp;
  gchar *gcHelp;
  Database::ASCIIDatabase::data *data = new Database::ASCIIDatabase::data;

  strhelp  = (gcHelp =xmms_remote_get_playlist_file(plugin->xmms_session, xmms_remote_get_playlist_pos(plugin->xmms_session)));
  if (gcHelp){
    g_free(gcHelp);
    gcHelp = NULL;
  }
  if (strhelp == "" ) {
    delete data;
    return;
  }

  if (inputNewAssessment == NULL) {
    delete data;
    return;
  }
  iNew = atoi(gtk_editable_get_chars(GTK_EDITABLE(inputNewAssessment), 0, -1));

  if(iNew > 1000 || iNew < 0){
    createValidValuesWindow();
    delete data;
    return;
  }


  data->setName(strhelp);
  data->setRate(iNew);

  if(db->modify(data) == DATABASECANNOTFINDENTRY) {
    db->write(data, 0, DATABASEAPPEND);
  }

   refreshConfigWindow(xmms_remote_get_playlist_pos(plugin->xmms_session));

   delete data;

} // end configWindowRateButtonClicked()










/*
  save the config values in default xmms config file
*/
int Config::saveValues() {
  
  char *cBuffer = new char[10];
  
  
  if (!(confFile = xmms_cfg_open_default_file())) 
    return CONFIGCANNOTOPEN;
  

   if(inputSkipSong != NULL) {
    cBuffer = gtk_editable_get_chars(GTK_EDITABLE(inputSkipSong), 0, -1);
    iRatingSongSkiped = atoi(cBuffer);
  }
  else
      sprintf(cBuffer, "%u", iRatingSongSkiped);
   xmms_cfg_write_string(confFile, "songRatePlaylist", "skipSongAway", cBuffer);


   if(inputHearedSong != NULL) {
    cBuffer = gtk_editable_get_chars(GTK_EDITABLE(inputHearedSong), 0, -1);
    iRatingSongHeared = atoi(cBuffer);
  }
  else
    sprintf(cBuffer, "%u", iRatingSongHeared);
  
  xmms_cfg_write_string(confFile, "songRatePlaylist", "hearedSong", cBuffer);
 


	
  iSongsRatedCountUntilNow += iSongsRatedCount;
  iSongsRatedCount = 0;

 
  if(inputTimeout != NULL){
    cBuffer = gtk_editable_get_chars(GTK_EDITABLE(inputTimeout), 0, -1);
    iTimeout = atoi(cBuffer);
  }
  else 
    sprintf(cBuffer, "%u", iTimeout);
 
  xmms_cfg_write_string(confFile, "songRatePlaylist", "timeout" ,cBuffer);


 
  if(inputDatabaseLocation != NULL) {
    cDatabaseLocation = gtk_editable_get_chars(GTK_EDITABLE(inputDatabaseLocation), 0, -1);
  }
 
  db->setLocation(cDatabaseLocation);

  xmms_cfg_write_string(confFile, "songRatePlaylist", "databaseLocation", cDatabaseLocation);
  


 
  xmms_cfg_write_default_file(confFile);
  xmms_cfg_free(confFile);
  confFile = NULL;
    
  free(cBuffer);
  
return CONFIGOK;
}// end configSave()


/*
  check if all config values are valid
*/
int Config::checkValues(){

  // save one call
  int help = 0;
  char *cBuffer;
  cBuffer = (char *) malloc(sizeof(int) *8 +1);
  string databaseLocation;
 
  // check input 1
  help = atoi(gtk_entry_get_text(GTK_ENTRY(inputSkipSong)));
  if(help > 1000 || help < 0){
    free(cBuffer); 
    createValidValuesWindow();
    return FALSE;
  }
 
  // aoti normalize char values
  // e.g. atoi("34%4")=>344
  // cause of this set text
  sprintf(cBuffer,"%d",help);
  gtk_entry_set_text(GTK_ENTRY(inputSkipSong), cBuffer);
  
  // check input 2
  help = atoi(gtk_entry_get_text(GTK_ENTRY(inputHearedSong)));
  if(help > 1000 || help < 0){
    free(cBuffer);
    createValidValuesWindow();
    return FALSE;
  }

  sprintf(cBuffer,"%d",help);
  gtk_entry_set_text(GTK_ENTRY(inputHearedSong), cBuffer);


  //check input 3
  if(!(help = atoi(gtk_entry_get_text(GTK_ENTRY(inputTimeout)))) || (help > 1000 || help < 0)){
    free (cBuffer);
    createValidValuesWindow();
    return FALSE;
  }
  sprintf(cBuffer,"%d", help);
  gtk_entry_set_text(GTK_ENTRY(inputTimeout), cBuffer);

  
  // check database
  databaseLocation = gtk_entry_get_text(GTK_ENTRY(inputDatabaseLocation));

  // ~/ support
  if(databaseLocation[0] == '~'){
    //delete ~
    databaseLocation.erase(0,1);
    databaseLocation.insert(0, getenv("HOME"));

    free(cBuffer);
    cBuffer=(char *) malloc ((int)databaseLocation.size()+1);
    strcpy(cBuffer,databaseLocation.c_str());

    gtk_entry_set_text(GTK_ENTRY(inputDatabaseLocation), cBuffer);
  }

  // check if file exist and create one if not
  ofstream file(gtk_entry_get_text(GTK_ENTRY(inputDatabaseLocation)), ios::out | ios::ate);
  if (!file) { 
    createFileNotFound();
    free(cBuffer);
    return FALSE;
  }
  else file.close();

  free(cBuffer);
return TRUE;
}// end configCheckValues()


void Config::createFileNotFound() {
  xmms_show_message("Configuration Error", "I couldn't open the database file\n\nPlease check your choosen database location and the read+write rights of its directory.", "OK", TRUE, NULL,  NULL);
} // end void Config::createFileNotFound()

void Config::createValidValuesWindow() {

  xmms_show_message("Configuration Error", "You entered an invalid Configuration.\nCorrect values are: 0-1000\nTry again ;)", "OK", TRUE, NULL,  NULL);

}// end configValidValuesWindow()




void Config::createRestoreReallyWindow() {

  GtkWidget *vbox, *hbox, *hbox2, *label, *button, *restoreReallyWin;
  // lovely copy&paste
   restoreReallyWin = gtk_window_new(GTK_WINDOW_DIALOG);
   gtk_signal_connect(GTK_OBJECT(restoreReallyWin), "destroy", GTK_SIGNAL_FUNC(gtk_widget_destroyed), &restoreReallyWin);
   gtk_window_set_title(GTK_WINDOW(restoreReallyWin), ("Restore Default Values??"));
   gtk_container_set_border_width (GTK_CONTAINER (restoreReallyWin), 25);
   

   vbox = gtk_vbox_new (FALSE, 15);
   gtk_container_add (GTK_CONTAINER (restoreReallyWin), vbox);
   
   hbox = gtk_hbox_new(FALSE, 15);
   gtk_container_add (GTK_CONTAINER (vbox), hbox);
     
 label = gtk_label_new(("Should I restore the default configuration values??"));
   gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
   gtk_widget_show(label);
		      
   gtk_widget_show(hbox);


   hbox2 = gtk_hbox_new(TRUE, 0);
   gtk_box_pack_start(GTK_BOX(vbox), hbox2, FALSE, FALSE, 0);

   hbox = gtk_hbox_new(TRUE, 15);
   gtk_box_pack_start(GTK_BOX(hbox2), hbox, FALSE, FALSE, 0);

   button = gtk_button_new_with_label(" YES ");
   gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, FALSE, 0);
   gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(st_restoreDefaultValues), this);
   gtk_signal_connect_object(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(restoreReallyWin));
   GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
   gtk_widget_show(button);

 button = gtk_button_new_with_label("NO!");
   gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, FALSE, 0);
   gtk_signal_connect_object(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(restoreReallyWin));
   GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
   gtk_widget_grab_default(button);

   gtk_widget_show(button);
   gtk_widget_show(hbox);
   gtk_widget_show(hbox2);
   gtk_widget_show(vbox);
		      
   gtk_widget_show(restoreReallyWin);

}// end configRestoreReallyWindow()



void Config::restoreDefaultValues(){
   gtk_entry_set_text (GTK_ENTRY (inputSkipSong), (gchar *)DEFAULT_ASSESSMENTSONGSKIPPED);
   gtk_entry_set_text (GTK_ENTRY (inputHearedSong), (gchar*)DEFAULT_ASSESSMENTSONGHEARED);
   gtk_entry_set_text(GTK_ENTRY(inputTimeout), (gchar *)DEFAULT_TIMEOUT);
   gtk_entry_set_text (GTK_ENTRY (inputDatabaseLocation), (gchar *)DEFAULT_DATABASELOCATION);
}//end configRestoreDefaultValues()




int Config::readValues() {

  char *chelp;
  int iSaveConfig = 0;

  // read global config values
  if (!(confFile = xmms_cfg_open_default_file())) 
    return CONFIGCANNOTOPEN;

  if(xmms_cfg_read_string(confFile,"songRatePlaylist", "skipSongAway" , &chelp)) {
    iRatingSongSkiped = atoi(chelp);
    }
  else iSaveConfig = 1;
    

  if (xmms_cfg_read_string(confFile,"songRatePlaylist", "hearedSong", &chelp))
    iRatingSongHeared = atoi(chelp);
  else iSaveConfig = 1;


  if (xmms_cfg_read_string(confFile, "songRatePlaylist", "timeout", &chelp))
    iTimeout = atoi(chelp);
  else iSaveConfig = 1;





  if(xmms_cfg_read_string(confFile, "songRatePlaylist", "databaseLocation", &chelp))
    cDatabaseLocation = chelp;
  else iSaveConfig = 1;
  
  
  xmms_cfg_free(confFile);

  if (iSaveConfig){
    saveValues();
     return CONFIGCANNOTREAD;
  }

return CONFIGOK;
}



/*
  ignore the current song in the future
*/
int Config::ignoreCurrentSong() {
  Database::ASCIIDatabase::data *data = new Database::ASCIIDatabase::data;
  string strhelp = "";
  int iret = 1;
  gchar *gcHelp;

  strhelp  = (gcHelp = xmms_remote_get_playlist_file(plugin->xmms_session, xmms_remote_get_playlist_pos(plugin->xmms_session)));
  
  if(gcHelp){
    g_free(gcHelp);
    gcHelp = NULL;
  }

  if (strhelp == "" ){
    delete data;
    return CONFIGOK;
  }
 
 data->setName(strhelp);
 data->setRate(0);

 if ((iret = db->modify(data)) == DATABASECANNOTFINDENTRY) {
       iret = db->write(data, 0, DATABASEAPPEND);
 }
 

 xmms_remote_playlist_next(plugin->xmms_session);

 
 delete data;

 return iret;
} // end ignoreCurrentSong()




int Config::refreshConfigWindow(int iPos) {

  int iret;
  char *cFile = NULL, *cTitle = NULL;
  long lPos = 0;
  char *cBuffer;
  string strhelp;
  Database::ASCIIDatabase::data *data = new Database::ASCIIDatabase::data;


  if ((iPos < 0) || (iPos >= xmms_remote_get_playlist_length(plugin->xmms_session))){
    delete data;
    return CONFIGERROR;
  }
  
  cTitle = xmms_remote_get_playlist_title(plugin->xmms_session, iPos);
  if (cTitle == NULL) {
    delete data;
    return CONFIGERROR;
  }
 
  cFile = xmms_remote_get_playlist_file(plugin->xmms_session, iPos);
  if (cFile == NULL) {
    delete data;
    return CONFIGERROR;
  }

  strhelp = (string) cFile;
  if(cFile) {
    g_free(cFile);
    cFile = NULL;
  }

  strhelp.erase(0, strhelp.rfind("/")+1);


  // refresh it!!
  if (inputCurrentSong != NULL) {

    gtk_entry_set_editable(GTK_ENTRY(inputCurrentSong), TRUE);

    gtk_entry_set_text (GTK_ENTRY (inputCurrentSong), cTitle);
    gtk_entry_set_editable(GTK_ENTRY(inputCurrentSong), FALSE);
  }
  if (inputCurrentSongAssessment != NULL) {

    switch(iret = db->search(&strhelp, lPos)) {
       case DATABASECANNOTFINDENTRY:
	 data->setRate();
	 break;
       case DATABASEOK:
	 db->read(*data, lPos);
	 break;
    default: 
      delete data;
      return iret;
    }

    cBuffer = (char*) malloc(sizeof(int) + 1);
    sprintf(cBuffer, "%d", data->getRate());
    gtk_entry_set_text (GTK_ENTRY (inputCurrentSongAssessment), cBuffer);
    free (cBuffer);
  }
  
  delete data;

return CONFIGOK;
}// end refreshConfigWindow()


void Config::createAboutWindow() {

  xmms_show_message("Rate & Ignore Plugin", "This Xmms Plugin trys to \
play always the songs you like most\n\n send all complaints & \
suggestions \
to:\nfidor@users.sourceforge.net\n\nhttp://xmmsassesment.sourceforge.net", "OK", TRUE, NULL, NULL);

}// end createAboutWindow()



#endif

