
#ifndef _RATEPLUG_DATABASE_H_
#define _RATEPLUG_DATABASE_H_


#include <string>
#include <fstream>





#define DATABASERATEMAX 1000


#define DATABASEOK 1

//failures
#define DATABASECANNOTOPEN -1
#define DATABASECANNOTREAD -2
#define DATABASECANNOTWRITE -3
#define DATABASECANNOTFINDENTRY -4
#define DATABASEERROR -5

// flags
#define DATABASEAPPEND 1


using namespace std;


class Database {
  
 public:
  class ASCIIDatabase{

  public:
    class data {
      string cName;
      int iRate;
      
    public:
      data() {
	iRate = DATABASERATEMAX/2;
	cName = "";
      }
      ~data() {
      }


      inline void setName(string _cName){
	unsigned int ihelp;

	cName = _cName;
       // just save the file name not the directory
	cName.erase(0, cName.rfind("/")+1);

	// remove whitespaces
	while((ihelp = cName.find(" ")) != string::npos) {
	  cName.replace(ihelp, 1, "_");
	}
      }
      inline string getName() {return cName;}

      inline void setRate(int _iRate = DATABASERATEMAX / 2){
	iRate = _iRate;
	iRate = (iRate > DATABASERATEMAX) ? DATABASERATEMAX : iRate;
	iRate = (iRate < 0) ? 0 : iRate;

      }
      inline int getRate() {return iRate;}
      

    };// end class data

  private:
    
    char *cDatabaseLocation;


  public:
    int setLocation(const char *cFileLocation);

    int search(string *pattern, long &lFilePos);



    int write(data *data, long lPos, int flags);
    int read(data &data, long lPos, int flags);

    int modify(data *data, long lPos = -1);
    int getMaxRating();
    int getMinRating();
    int length();



  };// end class ASCIIDatabase

};//end class database


/*
  implement ASCIIDatabase
*/

int Database::ASCIIDatabase::setLocation(const char* cFileLocation) {

  cDatabaseLocation = (char *)cFileLocation;

return DATABASEOK;
}// end setLocation()



int Database::ASCIIDatabase::search(string *pattern, long &lFilePos) {

   string strData;
   int iRate;
   unsigned int ihelp;


   if(*pattern == "") {
     return DATABASECANNOTFINDENTRY;
   }

   // delete whitespaces in pattern
   while((ihelp = pattern->find(" ")) != string::npos) {
     pattern->replace(ihelp, 1, "_");
   }
   
  ifstream file(cDatabaseLocation, ios::in);
  if (!file.is_open()) {
    return DATABASECANNOTOPEN;
    }
    while (1) {
      lFilePos = file.tellg();
      // if i'm not at the beginning of the file
      // i must go one further to access the next line
      if (lFilePos != 0) lFilePos++;
      // hier noch schaffen das strData gleich eine ganze Zeile enthaelt um ein wenig Overhead zu entfernen
      file>>strData>>iRate;

      if (!file.eof()) {
	if (strData == *pattern) {
	  file.close();
	  return DATABASEOK;
	}
	
      }
      else {
	file.close();
	lFilePos=-1;
	return DATABASECANNOTFINDENTRY;
      }

    }
    file.close();
return DATABASECANNOTFINDENTRY;
} // end databaseSearch()







int Database::ASCIIDatabase::write(data *data, long lPos, int flags = 0){

    // to save file integry
  string whitespaces = " ";
  int count = data->getRate();
  if (count == 0) count = 1;
  while (count < DATABASERATEMAX) {
    count *=10;
    whitespaces +=" ";
  }
  
  ofstream file(cDatabaseLocation, ios::out|ios::in);

  if (!file) { 
    return DATABASECANNOTOPEN;
  }


  // set file pointer
  if (!(flags & DATABASEAPPEND)){
    file.seekp(lPos);
  }
  else {
    /*    file.close();
    file.open(cDatabaseLocation, ios::out|ios::app);*/
    file.seekp(0,ios::end);
  }
  

  // fromated write 
  file << data->getName() << whitespaces << data->getRate()<<endl;

return DATABASEOK;
} // end databaseWrite()


int Database::ASCIIDatabase::read(data &data, long lPos, int flags = 0){
  
  string shelp;
  int ihelp;

  ifstream file(cDatabaseLocation, ios::in);
  if (!file) { 
    return DATABASECANNOTOPEN;
  }
  file.seekg(lPos);

  file >> shelp >> ihelp;

  data.setName(shelp);
  data.setRate(ihelp);

  return DATABASEOK;
}


int Database::ASCIIDatabase::modify(data *data, long lPos) {
  
 // get old rating
  if (lPos < 0) {
    if(search(&data->getName(), lPos) > 0){
      if(write(data, lPos) < 0) {
	return DATABASECANNOTWRITE;
      }
    }  
    else return DATABASECANNOTFINDENTRY;
  }
  else {
      if(write(data, lPos) < 0) {
	return DATABASECANNOTWRITE;
      }
  }


return DATABASEOK;

}// end databaseModifyRating()





int Database::ASCIIDatabase::getMaxRating() {

  string strData;
  int iRate, iMaxRate = 0;
  
  fstream file(cDatabaseLocation, ios::in);
  if (!file) {
    return DATABASECANNOTOPEN;
  }
  
  while (!file.eof()){
    file >>strData>>iRate;
 
    iMaxRate = (iRate > iMaxRate) ? iRate : iMaxRate;
  }

   if((iMaxRate < 0) || (iMaxRate > DATABASERATEMAX))
    return DATABASEERROR;

  return iMaxRate;
} // end databaseGetMaxRating()





int Database::ASCIIDatabase::getMinRating() {

  
  string strData;
  int iRate, iMinRate = DATABASERATEMAX;
  
  fstream file(cDatabaseLocation, ios::in);
  if (!file) {
    return DATABASECANNOTOPEN;
  }
  
  while (!file.eof()){
    file >>strData>>iRate;
     iMinRate = (iRate < iMinRate) ? iRate : iMinRate;
  }
  
  if((iMinRate < 0) || (iMinRate > DATABASERATEMAX))
    return DATABASEERROR;


  return iMinRate;
} // end databaseGetMinRating()



int Database::ASCIIDatabase::length(){
  int count=0, iRate;
  string strData;

  ifstream file(cDatabaseLocation, ios::in);
  if (!file) {
    return DATABASECANNOTOPEN;
  }

  while(!file.eof()){
    count++;
    file>>strData>>iRate;
  }
  return count;
}






#endif

